"""
bezier.py - Calculates a bezier curve from control points. 
 
Depends on the 2d vector class found here: http://www.gsdl2.org/wiki/2DVectorClass
 
2007 Victor Blomqvist
Released to the Public Domain
"""

from __future__ import print_function, division

import gsdl2
from gsdl2.locals import *

from vec2d import Vec2d
from bezier import calculate_bezier


SCREEN_SIZE = 800, 768
gray = Color(100, 100, 100)
lightgray = Color(200, 200, 200)
red = Color('red')
green = Color('green')
blue = Color('blue')
yellow = Color('yellow')
X, Y, Z = 0, 1, 2


def main():
    gsdl2.init()
    screen = gsdl2.display.set_mode(SCREEN_SIZE)
    rect = screen.get_rect()
    x, y, w, h = rect
 
    # Control points that are later used to calculate the curve
    control_points = [
        Vec2d(w*1/4, h*1/4), Vec2d(w*1/4, h*3/4),
        Vec2d(w*3/4, h*3/4), Vec2d(w*3/4, h*1/4),
    ]
    # convoy
    # the y position of the ranks in the standby formation
    _STANDBY_Y = [100, 150, 200, 250]
    control_points = [
        Vec2d(199, 0),
        Vec2d(220, 119), Vec2d(249, 248), Vec2d(284, 358),
        Vec2d(326, 465), Vec2d(406, 581), Vec2d(502, 581),
        Vec2d(594, 581), Vec2d(668, 523), Vec2d(662, 443),
        Vec2d(654, 361), Vec2d(547, 392), Vec2d(451, _STANDBY_Y[-1]+100)
    ]

    # The currently selected point
    selected = None
    
    clock = gsdl2.time.Clock()
    
    running = True
    while running:
        for e in gsdl2.event.get():
            if e.type == QUIT:
                running = False
            elif e.type == KEYDOWN:
                if e.scancode == S_ESCAPE:
                    running = False
                elif e.scancode == S_RETURN:
                    # try:
                        cp = control_points
                        print('{},'.format(cp[0]))
                        s = ''
                        for i in range(1, len(cp), 3):
                            for j in cp[i:i+3]:
                                s += '{},'.format(j)
                        print(s + '\n')
                    # except:
                    #     pass
                elif e.scancode == S_Z:
                    if len(control_points) > 2:
                        control_points.pop()
            elif e.type == MOUSEBUTTONDOWN and e.button == 1:
                for p in control_points:
                    if abs(p.x - e.pos[X]) < 10 and abs(p.y - e.pos[Y]) < 10:
                        selected = p
            elif e.type == MOUSEBUTTONDOWN and e.button == 3:
                x, y = gsdl2.mouse.get_pos()
                control_points.append(Vec2d(x, y))
            elif e.type == MOUSEBUTTONUP and e.button == 1:
                selected = None
        
        # Draw stuff
        screen.fill(gray)
                
        if selected is not None:
            selected.x, selected.y = gsdl2.mouse.get_pos()
            gsdl2.draw.circle(screen, green, selected, 10)
        
        # Draw control points
        for p in control_points:
            gsdl2.draw.circle(screen, blue, p, 4)
        # Draw control "lines"
        gsdl2.draw.lines(screen, lightgray, False, control_points)
        # Draw bezier curve
        for x in range((len(control_points) - 1) // 3):
            b_points = calculate_bezier(control_points[3*x:3*x+4])
#            gsdl2.draw.lines(screen, red, False, b_points)
            for bp in b_points:
                bp = tuple(int(p) for p in bp)
                gsdl2.draw.line(screen, yellow, bp, bp, 1)
        
        # Flip screen
        gsdl2.display.flip()
        clock.tick(100)
        # print clock.get_fps()

if __name__ == '__main__':
    main()
