"""credits.py - a context for displaying a game's credits
"""

import os
import random

import gsdl2

import context
import settings


class Credits(context.Context):
    
    def __init__(self):
        context.Context.__init__(self, gsdl2.time.FixedDriver(self.update, settings.time_step))
        self.clock.new_schedule(self.draw, 1.0 / 30.0)

        self.bgcolor = 0, 0, 0
        self.renderer = gsdl2.display.get_renderer()
        self.renderer.draw_color = self.bgcolor

        font_name, font_size = settings.font_info['ui']
        self.font = gsdl2.font.Font(font_name, 122)
        self.font_color = gsdl2.Color('gold')

        self.credits = None
        self.credits_rect = None
        self.elapsed = 0.0
        self.update(0)

    def update(self, dt):
        self.elapsed -= dt
        if self.elapsed <= 0.0:
            choice = random.choice
            choices = True, False
            self.elapsed += 1.0
            self.font.set_bold(choice(choices))
            self.font.set_italic(choice(choices))
            self.font.set_outline(choice(choices))
            self.font.set_underline(choice(choices))
            self.credits = gsdl2.texture.Texture(self.renderer, self.font.render('Gumm did it', self.font_color))
            self.credits_rect = self.credits.get_rect(center=settings.screen_rect.center)
        self.get_events()

    def draw(self, sched):
        self.renderer.clear()
        self.renderer.copy(self.credits, self.credits_rect)
        self.renderer.present()

    def on_key_down(self, e):
        self.on_quit()

    def on_mouse_button_down(self, e):
        self.on_quit()

    def on_user_event(self, e):
        if e.code == settings.song_finished:
            if settings.play_music:
                settings.music_play(0)

    def on_quit(self):
        self.pop()
