"""mainmenu.py - a context for running the application's main menu
"""

import os

import gsdl2

import context
import credits
import game
import settings


class ClickButton(object):
    
    def __init__(self, text_img, y, action):
        self.image = text_img
        self.rect = text_img.get_rect(y=y, centerx=settings.screen_rect.centerx)
        self.action = action
    
    def click(self, mouse_pos):
        if self.rect.collidepoint(mouse_pos):
            self.action()


class MainMenu(context.Context):
    
    def __init__(self):
        context.Context.__init__(self, gsdl2.time.FixedDriver(self.update, settings.time_step))
        self.clock.new_schedule(self.draw, 1.0 / 30.0)

        self.bgcolor = gsdl2.Color('darkgreen')
        self.renderer = gsdl2.display.get_renderer()
        self.renderer.draw_color = self.bgcolor
    
    def enter(self):
        font_name, font_size = settings.font_info['ui']
        font = gsdl2.font.Font(font_name, font_size)
        screen_centery = settings.screen_rect.centery
        text_height = font.get_height()
        
        self.buttons = []
        
        n = -1
        text_img = gsdl2.texture.Texture(self.renderer, font.render('PLAY GAME', gsdl2.Color('white')))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_play_game))
        
        n += 1
        text_img = gsdl2.texture.Texture(self.renderer, font.render('CREDITS', gsdl2.Color('white')))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_show_credits))
        
        n += 1
        text_img = gsdl2.texture.Texture(self.renderer, font.render('QUIT GAME', gsdl2.Color('white')))
        self.buttons.append(ClickButton(
            text_img, screen_centery + n * text_height, self.action_quit_game))

    def resume(self):
        self.renderer.draw_color = self.bgcolor

    def exit(self):
        gsdl2.music.set_endevent(None)
    
    def action_play_game(self):
        self.push(game.Game())
    
    def action_show_credits(self):
        self.push(credits.Credits())
    
    def action_quit_game(self):
        self.pop()
    
    def update(self, dt):
        self.get_events()

    def draw(self, sched):
        self.renderer.clear()
        for b in self.buttons:
            self.renderer.copy(b.image, b.rect)
        self.renderer.present()
    
    def on_mouse_button_down(self, e):
        for b in self.buttons:
            b.click(e.pos)

    def on_user_event(self, e):
        if e.code == settings.song_finished:
            if settings.play_music:
                settings.music_play(0)
